<?php

namespace Drupal\ephoto_dam\Plugin\Filter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use DOMDocument;

/**
 * Defines the plugin.
 *
 * @Filter(
 *   title = @Translation("Ephoto Dam"),
 *   id = "ephoto_dam_2",
 *   description = @Translation("Activate the use of the Ephoto Dam filter"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE
 * )
 */
class EphotoDam extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * Default settings.
   */
  private const DEFAULT_SETTINGS = [
    'ephoto_dam',
    'align',
    'src',
    'embed',
    'width',
    'height',
    'alt',
    'captions',
    'captions_format',
    'zoom',
  ];

  /**
   * Ephoto_dam constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin ID.
   * @param mixed $plugin_definition
   *   Plugin definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {

    $text = $this->replace($text);
    $result = new FilterProcessResult($text);
    $doc = new DOMDocument();
    $doc->loadHTML($text);
    $finder = new \DomXPath($doc);
    $classname = "ephoto_dam_iframe";
    $text = $doc->saveXML($doc);

    foreach ($this->getValidMatches($text) as $source => $values) {

      if (isset($values['ephoto_dam'])) {

        if ($values['ephoto_dam'] === 'image') {
          $html = $this->renderImage((object) $values);
        }

        else {
          $html = $this->renderEmbed((object) $values);
        }

        $text = str_replace($source, $html, $text);

        $result->setAttachments(['library' => ['ephoto_dam/display']]);
      }
    }

    $result->setProcessedText($text);

    return $result;
  }

  /**
   * Render Image code html.
   *
   * @param object $values
   *   Values of item.
   *
   * @return string
   *   Return image.
   */
  private function renderImage($values) {

    $html = '';

    if ($values->captions) {
      $html .= '<figure class="caption caption-img ephoto-dam-align-' . $values->align . '" role="group">';
    }

    $html .= '<article class="article_ephoto_dam media media--type-image media--view-mode-responsive-3x2 ephoto-dam-align-' . $values->align . '" style="width:' . $values->width . 'px">';
    $html .= '<img';
    $html .= ' alt="' . $values->alt . '"';
    $html .= ' src="' . $values->src . '"';

    if($values->ephoto_dam == "Text"){
      $values->width = $values->width*2;
    }

    $html .= 'style = width:' . $values->width . 'px;height:' . "auto" . '';

    $values->zoom = ($values->zoom === "true");
    $html .= ' class="' . ($values->zoom ? 'ephoto-dam-zoom' : '');
    $html .= '">';

    if ($values->caption) {
      $html .= '<figcaption class="ephoto_dam_caption">' . $values->caption . '</figcaption>';
    }

    $html .= '</article>';
    $html .= '</figure>';

    return $html;
  }

  /**
   * Render Embed code html.
   *
   * @param object $values
   *   Values of item.
   *
   * @return string
   *   Embeded media.
   */
  private function renderEmbed($values) {
    
    $html = '';

    if ($values->captions) {
      $html .= '<figure class="caption caption-img ephoto-dam-align-' . $values->align . '" role="group">';
    }

    $html .= '<artcle>';
    $html .= '<div style="width:' . $values->width . 'px; min-width:' . $values->width . 'px" class="ephoto_iframe align-' . $values->align . '">';
    $html .= '<iframe scrolling="no" allow="autoplay" allowfullscreen="true" frameborder="0"';
    $html .= ' alt="' . $values->alt . '"';
    $html .= ' src="' . $values->embed . '"';

    if ($values->newHeight != NULL) {
      $html .= ' height="' . $values->newHeight . '"';
    }
    else {
      $html .= ' height="' . $values->height . '"';
    }

    $html .= ' width="' . $values->width . '"';
    $html .= ' title="' . "test" . '"';
    $html .= '></iframe>';

    if ($values->caption) {
      $html .= '<figcaption class="ephoto_dam_caption">' . $values->caption . '</figcaption>';
    }

    $html .= "</div>";
    $html .= "</artcle>";
    $html .= '</figure>';

    return $html;
  }

  /**
   * Get all valid matches in the WYSIWYG.
   *
   * @param string $text
   *   The text to check for WYSIWYG matches.
   *
   * @return mixed
   *   An array of data from the text keyed by the text content.
   */
  protected function getValidMatches($text) {

    $return_valid_matches = [];

    $doc = new DOMDocument();
    $doc->loadHTML($text);
    $finder = new \DomXPath($doc);
    $classname = "ckeditor_ephoto_dam";
    $nodes = $finder->query("//*[contains(@class, '$classname')]");
    $i = 1;

    while ($i < $nodes->length + 1) {

      $node = $nodes[$i - 1];
      $nodeString = $node->ownerDocument->saveXML($node);
      $globalDivToReturn = $nodeString;
      $data = $node->firstElementChild->getAttribute("data");

      $array_value_tag = [
        "ephoto_dam",
        "src",
        "alt",
        "zoom",
        "oldwidth",
        "oldHeight",
        "embed",
        "width",
      ];

      $matches = [];
      foreach ($array_value_tag as $value_tag) {
        $split = explode($value_tag . ':{', $data);
        if (array_key_exists(1, $split)) {
          $split = $split[1];
          $split = explode('};', $split)[0];
          $matches[$value_tag] = $split;
        }
      }

      if ($matches["ephoto_dam"] === "") {
        return [];
      }

      $valid_matches = [];
      foreach ($matches as $delta => $match) {
        if ($this->isValidSettings($delta)) {
          $valid_matches[$delta] = $match;
        }
      }

      $matches["width"] = $node->firstElementChild->nextElementSibling->getAttribute("width");
      $matches["caption"] = $node->firstElementChild->nextElementSibling->getAttribute('data-caption');
      $matches["align"] = $node->firstElementChild->nextElementSibling->getAttribute('data-align');

      if ($matches["width"]) {
        $valid_matches["width"] = $matches["width"];
      }
      $valid_matches["caption"] = $matches["caption"];

      if ($matches["align"] != NULL) {
        $valid_matches["align"] = $matches["align"];
      }
      else {
        $valid_matches["align"] = "center";
      }

      if ($valid_matches['ephoto_dam'] != "image") {

        $split = explode('title:{', $data);
        if (array_key_exists(1, $split)) {
          $split = $split[1];
          $split = explode('};', $split)[0];
        }
        $valid_matches['title'] = $split;

        $split = explode('height:{', $data);
        if (array_key_exists(1, $split)) {
          $split = $split[1];
          $split = explode('};', $split)[0];
        }
        $valid_matches['oldheight'] = $split;

        $split = explode('width:{', $data);
        if (array_key_exists(1, $split)) {
          $split = $split[1];
          $split = explode('};', $split)[0];
        }
        $valid_matches['oldwidth'] = $split;

        $url_components = parse_url($valid_matches["embed"]);
        parse_str($url_components['query'], $params);

        if ($valid_matches["width"] != NULL) {
          $valid_matches["newHeight"] = intval(($valid_matches["width"] * $valid_matches["oldheight"]) / $valid_matches["oldwidth"]);
        }
        $valid_matches['embed'] = str_replace("width=" . $params['width'], "width=" . $valid_matches['width'], $valid_matches["embed"]);
        $valid_matches["embed"] = str_replace("&height=" . $params['height'], "&height=" . $valid_matches['newHeight'], $valid_matches["embed"]);
      }

      $nodeString = $node->ownerDocument->saveXML($node);
      $globalDivToReturn = $nodeString;

      $divMedia = $globalDivToReturn;
      $return_valid_matches[$divMedia] = $valid_matches;
      $i++;
    }

    return $return_valid_matches;
  }

  /**
   * Check if the given settings are valid.
   *
   * @param mixed $settings
   *   Settings to validate.
   *
   * @return bool
   *   If the required settings are present.
   */
  protected function isValidSettings($settings) {
    return in_array($settings, self::DEFAULT_SETTINGS);
  }

  /**
   * Replace a string with a special characters free one.
   *
   * @param string $name
   *   The string to transform.
   *
   * @return string
   *   The transformed string
   */
  private function replace($name) {

    $name = str_replace("é", "e", $name);
    $name = str_replace("è", "e", $name);
    $name = str_replace("ê", "e", $name);
    $name = str_replace("à", "a", $name);
    $name = str_replace("â", "a", $name);
    $name = str_replace("î", "i", $name);
    $name = str_replace("ô", "o", $name);
    $name = str_replace("û", "u", $name);

    $name = str_replace("É", "E", $name);
    $name = str_replace("È", "E", $name);
    $name = str_replace("Ê", "E", $name);
    $name = str_replace("À", "A", $name);
    $name = str_replace("Â", "A", $name);
    $name = str_replace("Î", "I", $name);
    $name = str_replace("Ô", "O", $name);
    $name = str_replace("Û", "U", $name);
    return $name;
  }

}
